BEGIN;

CREATE TABLE IF NOT EXISTS notebook (
	object_id TEXT UNIQUE NOT NULL,
	category TEXT DEFAULT 'notebook' NOT NULL,
	title TEXT NOT NULL,
	preset BOOLEAN DEFAULT FALSE,
	acl JSON,
	owner BIGINT NOT NULL,
	version TEXT NOT NULL,
	ctime INTEGER,
	mtime INTEGER,
	link_id TEXT NOT NULL,
	individual_shared BOOLEAN DEFAULT FALSE,
	extra_info JSON
);

CREATE INDEX idx_notebook_object_id ON notebook(object_id);
CREATE INDEX idx_notebook_ctime ON notebook(ctime);
CREATE INDEX idx_notebook_mtime ON notebook(mtime);

CREATE TABLE IF NOT EXISTS note (
	object_id TEXT UNIQUE NOT NULL,
	category TEXT DEFAULT 'note' NOT NULL,
	latitude REAL,
	longitude REAL,
	location TEXT,
	title TEXT NOT NULL,
	brief TEXT,
	acl JSON,
	owner BIGINT NOT NULL,
	version TEXT NOT NULL,
	ctime INTEGER,
	mtime INTEGER,
	parent_id TEXT NOT NULL,
	encrypt BOOLEAN DEFAULT FALSE,
	recycle BOOLEAN DEFAULT FALSE,
	thumb TEXT,
	tag JSON,
	attachment JSON,
	old_parent_id TEXT,
	link_id TEXT NOT NULL,
	individual_shared BOOLEAN DEFAULT FALSE,
	extra_info JSON
);

CREATE INDEX idx_note_object_id ON note(object_id);
CREATE INDEX idx_note_title ON note(title);
CREATE INDEX idx_note_ctime ON note(ctime);
CREATE INDEX idx_note_mtime ON note(mtime);
CREATE INDEX idx_note_parent_id ON note(parent_id);
CREATE INDEX idx_note_recycle_and_owner ON note(recycle, owner);

CREATE TABLE IF NOT EXISTS note_encrypt (
	object_id TEXT NOT NULL,
	ctime INTEGER NOT NULL,
	mtime INTEGER NOT NULL,
	duration INTEGER NOT NULL,
	token TEXT UNIQUE NOT NULL,
	password TEXT NOT NULL
);

CREATE INDEX idx_note_encrypt_object_id ON note_encrypt(object_id);

CREATE TABLE IF NOT EXISTS link (
	id TEXT UNIQUE NOT NULL,
	object_id TEXT NOT NULL,
	category TEXT NOT NULL
);

CREATE INDEX idx_link_id ON link(id);
CREATE INDEX idx_link_object_id ON link(object_id);

CREATE TABLE IF NOT EXISTS permission (
	object_id TEXT NOT NULL,
	uid BIGINT NOT NULL,
	perm INTEGER NOT NULL,
	listable BOOLEAN NOT NULL,
	inherit BOOLEAN NOT NULL
);

CREATE INDEX idx_permission_object_id ON permission(object_id);
CREATE INDEX idx_permission_object_id_and_uid ON permission(object_id, uid);
CREATE INDEX idx_permission_perm ON permission(perm);

CREATE TABLE IF NOT EXISTS tag_note (
	note_object_id TEXT NOT NULL,
	tag_id TEXT NOT NULL
);

CREATE INDEX idx_tag_note_object_id ON tag_note(note_object_id);
CREATE INDEX idx_tag_note_tag_title ON tag_note(tag_id);
CREATE INDEX idx_tag_note_object_id_and_tag_title ON tag_note(note_object_id, tag_id);

CREATE TABLE IF NOT EXISTS tag (
	id TEXT UNIQUE NOT NULL,
	title TEXT NOT NULL,
	category TEXT DEFAULT 'tag' NOT NULL,
	owner BIGINT NOT NULL,
	extra_info JSON
);

CREATE INDEX idx_tag_title ON tag(title);

CREATE TABLE IF NOT EXISTS object_shortcut (
	owner BIGINT NOT NULL,
	object_id TEXT NOT NULL
);

CREATE INDEX idx_object_shortcut_object_id ON object_shortcut(object_id);

CREATE TABLE IF NOT EXISTS tag_shortcut (
	owner BIGINT NOT NULL,
	tag_id TEXT NOT NULL
);

CREATE INDEX idx_tag_shortcut_tag_id ON tag_shortcut(tag_id);

CREATE OR REPLACE FUNCTION afterNotebookDeleted()
RETURNS "trigger" AS $$
	BEGIN
		DELETE FROM permission WHERE object_id=OLD.object_id;
		DELETE FROM object_shortcut WHERE object_id=OLD.object_id;
		DELETE FROM link WHERE object_id=OLD.object_id;
		RETURN NULL;
	END;
$$ LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION afterNoteDeleted()
RETURNS "trigger" AS $$
	BEGIN
		DELETE FROM tag_note WHERE note_object_id=OLD.object_id;
		DELETE FROM permission WHERE object_id=OLD.object_id;
		DELETE FROM object_shortcut WHERE object_id=OLD.object_id;
		DELETE FROM link WHERE object_id=OLD.object_id;
		DELETE FROM note_encrypt WHERE object_id=OLD.object_id;
		RETURN NULL;
	END;
$$ LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION afterTagRelationDeleted()
RETURNS "trigger" AS $$
	BEGIN
		DELETE FROM tag
		WHERE 0=(SELECT count(tag_id) FROM tag_note WHERE tag_id=OLD.tag_id)
		AND id=OLD.tag_id;
		RETURN NULL;
	END;
$$ LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION afterTagDeleted()
RETURNS "trigger" AS '/var/packages/NoteStation/target/trigger/pgtrigger.so', 'tag_deleted'
LANGUAGE C;

CREATE TRIGGER "onDelete" AFTER DELETE
   ON notebook
   FOR EACH ROW
   EXECUTE PROCEDURE afterNotebookDeleted();

CREATE TRIGGER "onDelete" AFTER DELETE
   ON note
   FOR EACH ROW
   EXECUTE PROCEDURE afterNoteDeleted();

CREATE TRIGGER "onDelete" AFTER DELETE
   ON tag_note
   FOR EACH ROW
   EXECUTE PROCEDURE afterTagRelationDeleted();

CREATE TRIGGER "onDelete" AFTER DELETE
   ON tag
   FOR EACH ROW
   EXECUTE PROCEDURE afterTagDeleted();

COMMIT;
