/**
 * Require the given path.
 *
 * @param {String} path
 * @return {Object} exports
 * @api public
 */

function require(p, parent, orig){
	var path = require.resolve(p)
		, mod = require.modules[path];

	// lookup failed
	if (null == path) {
		orig = orig || p;
		parent = parent || 'root';
		throw new Error('failed to require "' + orig + '" from "' + parent + '"');
	}

	// perform real require()
	// by invoking the module's
	// registered function
	if (!mod.exports) {
		mod.exports = {};
		mod.client = mod.component = true;
		mod.call(this, mod, mod.exports, require.relative(path));
	}

	return mod.exports;
}

/**
 * Registered modules.
 */

require.modules = {};

/**
 * Registered aliases.
 */

require.aliases = {};

/**
 * Resolve `path`.
 *
 * Lookup:
 *
 *   - PATH/index.js
 *   - PATH.js
 *   - PATH
 *
 * @param {String} path
 * @return {String} path or null
 * @api private
 */

require.resolve = function(path){
	var orig = path
		, reg = path + '.js'
		, regJSON = path + '.json'
		, index = path + '/index.js'
		, indexJSON = path + '/index.json';

	return require.modules[reg] && reg
		|| require.modules[regJSON] && regJSON
		|| require.modules[index] && index
		|| require.modules[indexJSON] && indexJSON
		|| require.modules[orig] && orig
		|| require.aliases[index];
};

/**
 * Normalize `path` relative to the current path.
 *
 * @param {String} curr
 * @param {String} path
 * @return {String}
 * @api private
 */

require.normalize = function(curr, path) {
	var segs = [];

	if ('.' != path.charAt(0)) return path;

	curr = curr.split('/');
	path = path.split('/');

	for (var i = 0; i < path.length; ++i) {
		if ('..' == path[i]) {
			curr.pop();
		} else if ('.' != path[i] && '' != path[i]) {
			segs.push(path[i]);
		}
	}

	return curr.concat(segs).join('/');
};

/**
 * Register module at `path` with callback `fn`.
 *
 * @param {String} path
 * @param {Function} fn
 * @api private
 */

require.register = function(path, fn){
	require.modules[path] = fn;
};

/**
 * Alias a module definition.
 *
 * @param {String} from
 * @param {String} to
 * @api private
 */

require.alias = function(from, to){
	var fn = require.modules[from];
	if (!fn) throw new Error('failed to alias "' + from + '", it does not exist');
	require.aliases[to] = from;
};

/**
 * Return a require function relative to the `parent` path.
 *
 * @param {String} parent
 * @return {Function}
 * @api private
 */

require.relative = function(parent) {
	var p = require.normalize(parent, '..');

	/**
	 * lastIndexOf helper.
	 */

	function lastIndexOf(arr, obj){
		var i = arr.length;
		while (i--) {
			if (arr[i] === obj) return i;
		}
		return -1;
	}

	/**
	 * The relative require() itself.
	 */

	function fn(path){
		var orig = path;
		path = fn.resolve(path);
		return require(path, parent, orig);
	}

	/**
	 * Resolve relative to the parent.
	 */

	fn.resolve = function(path){
		// resolve deps by returning
		// the dep in the nearest "deps"
		// directory
		if ('.' != path.charAt(0)) {
			var segs = parent.split('/');
			var i = lastIndexOf(segs, 'deps') + 1;
			if (!i) i = 0;
			path = segs.slice(0, i + 1).join('/') + '/deps/' + path;
			return path;
		}
		return require.normalize(p, path);
	};

	/**
	 * Check if module is defined at `path`.
	 */

	fn.exists = function(path){
		return !! require.modules[fn.resolve(path)];
	};

	return fn;
};

require.register("itsjoesullivan-event/index.js", function(module, exports, require){
module.exports = function() {};

/**
 * Add a listener by event name
 * @param {String} name
 * @param {Function} fn
 * @return {Event} instance
 * @api public
 */
module.exports.prototype.on = function(name,fn) {

	//Lazy instanciation of events object
	var events = this.events = this.events || {};

	//Lazy instanciation of specific event
	events[name] = events[name] || [];

	//Give it the function
	events[name].push(fn);

	return this;

};


/**
 * Trigger an event by name, passing arguments
 *
 * @param {String} name
 * @return {Event} instance
 * @api public
 */
module.exports.prototype.trigger = function(name, arg1, arg2 /** ... */) {

	//Only if events + this event exist...
	if(!this.events || !this.events[name]) return this;

	//Grab the listeners
	var listeners = this.events[name],
		//All arguments after the name should be passed to the function
		args = Array.prototype.slice.call(arguments,1);

	//So we can efficiently apply below
	function triggerFunction(fn) {
		fn.apply(this,args);
	};

	if('forEach' in listeners) {
		listeners.forEach(triggerFunction.bind(this));
	} else {
		for(var i in listeners) {
			if(listeners.hasOwnProperty(i)) triggerFunction(fn);
		}
	}

	return this;

};
});

require.register("itsjoesullivan-thread/index.js", function(module, exports, require){
var Event = require('event');

/** Create a new thread
*/
var Thread = function(workerObject) {

	//Convert worker object to a string
	var objectString = this.stringifyObject(workerObject);

	//Turn it into an immediately-executing function
	var workerString = this.encloseString(objectString);

	var workerWithHelper = this.prependHelpers(workerString);

	//Make a blob of it
	var blob = new Blob([workerWithHelper], {type: 'text/javascript'});

	//Create a worker pointed at the blob
	this.worker = new Worker(window.URL.createObjectURL(blob));

	this.worker.addEventListener('message', function(e) {
		if(typeof e.data === 'object' && 'type' in e.data && e.data.type === 'thread.send') {
			this.trigger.apply(this,[e.data.name].concat(e.data.args));
		} else {
			this.trigger('message',e.data,e);
		}
	}.bind(this));

	return this;
};

Thread.prototype = new Event();

Thread.prototype.send = function(name,arg1,arg2 /* ... */) {
	var obj = {
		type: 'thread.send',
		name: name,
		args: Array.prototype.slice.call(arguments,1)
	};
	this.worker.postMessage.apply(this.worker,[obj]);
};

/**
 * Turn object into string.
 */
Thread.prototype.stringifyObject = function(workerObject) {
	if(typeof workerObject === 'function') {
		return workerObject.toString();
	}
	var workerString = '{\n';
	for(var i in workerObject) {
		workerString += '\t' + i + ': ' + this.stringifyObject(workerObject[i]) + ',\n';
	}
	workerString = workerString.substring(0,workerString.length-2) + '\n';
	workerString += '};';
	return workerString;
};

/**
 * Turn string into immediately-executing function string
 */
Thread.prototype.encloseString = function(workerString) {

	return '(' + workerString + ')()';

};

Thread.prototype.prependHelpers = function(workerString) {

	var helpers = 'onmessage = ' + function(e) {
		if(e.data.type === 'thread.send') {
			thread.trigger(e.data.name,e.data.args);
		} else {
			postMessage(e.data);
		}
	}.toString() + '\n' +
	'thread = {' +
		'send: ' + function(name,arg1,arg2) {
			postMessage({
				name: name,
				args: Array.prototype.slice.call(arguments,1),
				type: 'thread.send'
			});
		}.toString() + ', \n' +
		'on: ' + Event.prototype.on.toString() + ',\n' +
		'trigger: ' + Event.prototype.trigger.toString() + '\n' +
	'};\n';

	return helpers + '\n' + workerString;
};

module.exports = Thread;
});

require.register("iago/index.js", function(module, exports, require){
var Thread = require('thread');
var RecordState = {
	STOP: 0,
	START: 1,
	PAUSE: 2
};

var syno_recorder_mgr = {
	audio_context: null
};

var Iago = module.exports = function(options) {
	var succ, error;

	this.options = options || {};
	if (!syno_recorder_mgr.audio_context) {
		syno_recorder_mgr.audio_context = new AudioContext();
	}
	this.audio_context = syno_recorder_mgr.audio_context;
	this.stream = null;
	this.record_state = RecordState.STOP;
	this.accu_time = 0;

	succ = function (handle) {
		this.stream = this.audio_context.createMediaStreamSource(handle);
		if (!handle instanceof window.MediaStreamAudioSourceNode) {
			return false;
		}

		this.source = this.stream;
		this.context = this.source.context;

		if (Object.prototype.toString.call(options.success) == '[object Function]') {
			options.success.apply(options.scope, arguments);
		}
	};

	error = function (event) {
		if (Object.prototype.toString.call(options.fail) == '[object Function]') {
			options.fail.apply(options.scope, arguments);
		}
	};

	navigator.getUserMedia({
			audio: true,
			video: false
		},
		succ.bind(this),
		error.bind(this)
	);
};

Iago.prototype.record = function() {
	var url;

	if (this.record_state !== RecordState.STOP || !this.source || !this.context) {
		return false;
	}

	if (this.thread) {
		delete this.thread;
	}

	this.record_state = RecordState.START;

	if (!this.input) {
		this.input = this.context.createScriptProcessor(4096, 2, 2);
		this.input.onaudioprocess = this.write.bind(this);
		this.input.connect(this.context.destination);
		this.source.connect(this.input);
	}

	this.thread = new Thread(function() {
		var state;

		thread.on('init', function (url) {
			// Import vorbis into worker
			importScripts(url + '/vorbis.js');

			// Initialize
			state = Module._lexy_encoder_start(44100, 8);
		});

		// On writes, write to vorbis
		thread.on('write', function(data) {
			if (!state) {
				return;
			}
			var l = data[0];
			var r = data[1];

			var inbuf_l = Module._malloc(l.length*l.BYTES_PER_ELEMENT);
			Module.HEAPF32.set( l, inbuf_l>>2 );

			var inbuf_r = Module._malloc(r.length*r.BYTES_PER_ELEMENT);
			Module.HEAPF32.set( r, inbuf_r>>2 );

			Module._lexy_encoder_write( state, inbuf_l, inbuf_r, l.length );

			Module._free(inbuf_l);
			Module._free(inbuf_r);

		});

		thread.on('finish', function(e) {

			// Wrap up
			Module._lexy_encoder_finish( state );

			// Get pointer
			var ptr = Module._lexy_get_buffer( state );

			var oggData = new Uint8Array( Module.HEAPU8.subarray( ptr,
				ptr + Module._lexy_get_buffer_length( state ) )
			);
			thread.send('blob', oggData);

		});
	});

	url = this.options.url || window.location.origin + '/iago/vorbis.js';
	this.thread.send('init', url);
	this.start_time = null;
};

Iago.prototype.write = function( event ) {
	var diff;

	if (this.record_state !== RecordState.START) {
		// No more writes"
		return;
	}

	if (null === this.start_time) {
		this.start_time = new Date().getTime();
	}

	if (Object.prototype.toString.call(this.options.progress) == '[object Function]') {
		diff = new Date().getTime() - this.start_time + this.accu_time;
		this.options.progress.call(this.scope, {
			timer: diff
		});
	}

	this.thread.send('write',
		event.inputBuffer.getChannelData(0),
		event.inputBuffer.getChannelData(1)
	);

};

Iago.prototype.isRecording = function () {
	return this.record_state === RecordState.START;
}

Iago.prototype.pause = function () {
	this.accu_time = new Date().getTime() - this.start_time + this.accu_time;
	this.start_time = null;
	this.record_state = RecordState.PAUSE;
}

Iago.prototype.resume = function () {
	if (this.record_state === RecordState.PAUSE) {
		this.start_time = new Date().getTime();
		this.record_state = RecordState.START;
	}
}

Iago.prototype.stop = function (callback, scope) {
	this.record_state = RecordState.STOP;

	this.start_time = null;
	this.accu_time = 0;

	if (this.blob) {
		if (Object.prototype.toString.call(callback) == '[object Function]') {
			callback.call(scope, this.blob);
		}
		return;
	}

	this.thread.on('blob', function( data ) {
		this.blob = new Blob( [ data ], {
			type: 'audio/ogg'
		});

		if (Object.prototype.toString.call(callback) == '[object Function]') {
			callback.call(scope, this.blob);
		}
	}.bind(this));

	this.thread.send('finish');
}

Iago.prototype.getBlob = function() {

	if (!this.blob) {
		return false;
	}

	return this.blob;
};

Iago.prototype.download = function(targetWin) {

	if (this.record_state !== RecordState.STOP || !this.blob) {
		return false;
	}

	targetWin = targetWin || window;

	// Download
	var a = targetWin.document.createElement('a');
	a.href = targetWin.window.URL.createObjectURL(this.blob);
	a.download = 'test.ogg';
	targetWin.document.body.appendChild(a);
	a.click();

	return true;

};

Iago.prototype.clear = function () {

	if (!this.source) {
		return;
	}

	this.stop();
	if (this.source.mediaStream) {
		this.source.mediaStream.stop();
	}
	this.audio_context = null;
	this.source = null;
	this.context = null;
	this.record_state = RecordState.STOP;
	this.blob = null;
};

});
require.alias("itsjoesullivan-thread/index.js", "iago/deps/thread/index.js");
require.alias("itsjoesullivan-event/index.js", "itsjoesullivan-thread/deps/event/index.js");
require.alias("iago/index.js", "iago/index.js");

window.AudioContext = window.AudioContext || window.webkitAudioContext;
navigator.getUserMedia = (navigator.getUserMedia || navigator.webkitGetUserMedia || navigator.mozGetUserMedia || navigator.msGetUserMedia);
window.Iago = require('iago');
